# Upgrade Kamailio from v5.5.x to v5.6.0 (pre)

The page contains the details about the changes that were made to old
components during the development of `v5.6.0` compared with what existed
in `v5.5.x`. It does not include the brand new modules, focusing on how to
upgrade database and configuration file from `v5.5.x` to run with Kamailio
`5.6.0`.

## Previous Stable Release

If you look for the guidelines to upgrade to previous stable release,
see:

- [Upgrade Kamailio v5.4.x to v5.5.0](5.4.x-to-5.5.0.md)

## Remarks

Following tokens are used to mark the changes:

- `INF` - the change doesn't really have any direct impact - no action
    required in old config
- `CPM` - the change was used with the new value via explicit parameter
    in default config file from old releases
- `RCM` - the change is recommended to be done if you had an explicit
    different or lower value for this parameter in old config
- `UPG` - the change has to be done, older config does not work any
    longer

## Modules

### Upgraded Modules

#### dispatcher

- rpc command to add a record updated to set priority

#### presence

- return `200` instead of `202` for SUBSCRIBE by default (RFC6665)

#### siputils

- removed `ring_insert_callid()` function (see: <https://github.com/kamailio/kamailio/issues/2989>)

#### uac_redirect

- `acc_function` mod param has to be set to empty string to disable binding to `acc` module

### Removed Modules

- none

### Renamed Modules

- `malloc_test` - replaced by `misctest` (`UPG`)

## Internal Libraries

## Core

- intensive C code fuzzing and static analysis were performed which resulted in
adding stricker checks for input values. That can cause change in behaviour
for some variables or transformations. For example, `{s.int}` checks that
the value is in between INT_MIN and INT_MAX.

### Parameters

#### Dst Blocklist Parameters

(`UPG`)

All global parameters related to dst blocking are now using blocklist
instead of blacklist in the name, see:

- [Core Cookbook](../../cookbooks/5.6.x/core.md#blocklist_parameters)

For example, **use_dst_blacklist** used in older versions has to be
replaced by **use_dst_blocklist**.

### Misc

## Database

### MySQL Database Structure

The following changes were done to the database tables existing in v5.5.x:

- (`INF`, `RCM`) index was created on column `connection_id` for table `location` of `usrloc` module
- (`UPG`) type of column `protocol` for table `location` of `ims_usrloc_pcscf` module was changed from `char(5)` to `int(10)` (used only for IMS deployments)

Run following SQL statements in MySQL client to upgrade database
structure from v5.5 to v5.6:

``` sql
-- add index on connection_id for usrloc module location table
CREATE INDEX tcpcon_idx ON location (`connection_id`);
```
